import numpy as np
import cv2

class TextureExtractor:
    def __init__(self) -> None:
        pass

    def extract_boundary(self, point_cloud):
        projected_points = point_cloud[:, :2]  # Keep only the X and Y coordinates

        # Sort the points in a clockwise direction
        center = np.mean(projected_points, axis=0)
        sorted_points = sorted(projected_points, key=lambda p: np.arctan2(p[1] - center[1], p[0] - center[0]))
        sorted_points = np.array(sorted_points, dtype=np.float32)

        hull = cv2.convexHull(sorted_points, clockwise=True)
        # Create a boundary list
        boundary = []
        
        if len(hull) > 0:
            for point_bd in hull:
                bd_x, bd_y = point_bd[0].astype(int)
                boundary.append([bd_x, bd_y])
        return np.asarray(boundary)
    

    def extract_face_area(self, image, boundary_points):
        height, width, _ = image.shape
        mask = np.zeros((height, width), dtype=np.uint8)

        cv2.fillPoly(mask, [boundary_points], 255)  
        face_area = cv2.bitwise_and(image, image, mask=mask)
        return face_area


    def get_projected_points(self, point_cloud, fx=10.0, fy=10.0, cx=450, cy=450):
        projected_points = []  # List to store 2D pixel coordinates

        for point in point_cloud:
            x, y, z = point
            u = int(fx * x / z + cx)
            v = int(fy * y / z + cy)
            projected_points.append((u, v))
        return projected_points
    

    def calculate_u_coordinate(self, point, x_min, x_max):
        # Assuming the X coordinate of the point is in the range [x_min, x_max]
        x = point[0]

        u = (x - x_min) / (x_max - x_min)
        u = max(0.0, min(1.0, u))
        return u


    def calculate_v_coordinate(self, point, y_min, y_max):
        # Assuming the Y coordinate of the point is in the range [y_min, y_max]
        y = point[1]

        v = (y - y_min) / (y_max - y_min)
        v = max(0.0, min(1.0, v))
        return v
    

    def extract_texture(self, original_image, point_cloud, square_size = 256):
        uv_coordinates = []  # List to store 2D UV texture coordinates
        original_image_height, original_image_width, _ = original_image.shape

        x_min = min(point[0] for point in point_cloud)
        x_max = max(point[0] for point in point_cloud)
        y_min = min(point[1] for point in point_cloud)
        y_max = max(point[1] for point in point_cloud)

        for point in point_cloud:
            # Calculate UV coordinates based on the properties of your 3D model
            u = self.calculate_u_coordinate(point, x_min, x_max)
            v = self.calculate_v_coordinate(point, y_min, y_max)
            uv_coordinates.append((u, v))

        texture_image = np.zeros((original_image_height, original_image_width, 3), dtype=np.uint8)

        for u, v in uv_coordinates:
            x = int(u * (original_image_width - 1))  # Map u to image width
            y = int(v * (original_image_height - 1))  # Map v to image height

            if 0 <= x < original_image_width and 0 <= y < original_image_height:
                texture_image[y, x] = original_image[y, x]

        texture_image = cv2.resize(texture_image, (square_size, square_size))
        return texture_image















import os
import cv2
import argparse
import numpy as np

from medface3D.worker.face_mesh_reconstruction import FaceMeshGenerator
from medface3D.utils.render_mesh import render_mesh
from medface3D.utils.transform import get_3D_point_cloud
from medface3D.simualte.simuator import merge_action


def main():
    # Parse command-line arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-v',  '--video_path',  default=0 , help='path to sample folder image')
    parser.add_argument('-l',  '--max_loop',    type=int, default=0 ,   help='max loop to reconstruct 3D face mesh')

    args = parser.parse_args()

    detector = FaceMeshGenerator(max_loop = args.max_loop)
    cap = cv2.VideoCapture(args.video_path)

    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    size = (int(cap.get(cv2.CAP_PROP_FRAME_WIDTH)) * 3, int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT)))

    if args.video_path == 0:
        name_save = 'local.mp4'
    else: name_save = os.path.basename(args.video_path)

    out = cv2.VideoWriter(f'data/demo/mesh_{name_save}', fourcc, 10, size)
    simulator_des = cv2.imread(f'data/sample/merge_action.jpg')
    output_image, _, points, depth_list, _ = detector.generate_face_mesh(simulator_des, True, False)

    des_object_pcd = get_3D_point_cloud(points, depth_list)

    if not cap.isOpened():
        print("Error: Could not open camera.")
        exit()

    while True:
        ret, frame = cap.read()

        # Check if the frame was captured successfully
        if not ret:
            print("Error: Could not read frame.")
            break
        
        input_image = frame
        output_image, face_detected, points, depth_list, _ = detector.generate_face_mesh(input_image, True, False)
        
        if face_detected:
            point_cloud_data = get_3D_point_cloud(points, depth_list)

            transformed_obj = merge_action(point_cloud_data, des_object_pcd)
            mesh_blank = render_mesh(input_image, transformed_obj)
            output_image = cv2.hconcat([frame, mesh_blank])

        out.write(output_image)
        # Display the frame in a window
        cv2.imshow("Video Capture", output_image)

        # Exit the loop when the 'q' key is pressed (or any other key)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    # Release the VideoCapture object and close the OpenCV window
    cap.release()
    out.release()
    cv2.destroyAllWindows()

if __name__ == '__main__':
    # Example usage:
    main()



import numpy as np
import cv2
import scipy.spatial
from utils.transform import get_matrix_rotate_point_around_x, \
                            get_matrix_rotate_point_around_y, \
                            get_matrix_rotate_point_around_z

def render_mesh(image_ori, point_cloud_data, blank_background = True):
    image_size = (image_ori.shape[1], image_ori.shape[0])
    
    if not blank_background:
        image = image_ori.copy()
    else: image = np.zeros((image_size[1], image_size[0], 3), dtype=np.uint8)

    # Define a simple light direction (you can adjust this as needed)
    light_direction = np.array([0, 0, 1])
    # Create a Delaunay triangulation to connect the points and form a mesh
    triangles = scipy.spatial.Delaunay(point_cloud_data[:, :2]) 
    
    for simplex in triangles.simplices:
        vertices = point_cloud_data[simplex]

        # Compute the face normal (in this case, we use the cross product of two edges)
        edge1 = vertices[1] - vertices[0]
        edge2 = vertices[2] - vertices[0]

        face_normal = np.cross(edge1, edge2)
        face_normal /= np.linalg.norm(face_normal)

        # Calculate the intensity based on the dot product with the light direction
        intensity = np.dot(face_normal, light_direction)

        # Clip the intensity to avoid negative values
        intensity = max(intensity, 0)
        color=(intensity * 200, intensity * 255, intensity * 50)

        # Draw the triangle with shading
        cv2.fillPoly(image, [vertices[:, :2].astype(int)], color = color)
    return image


def calculate_center(point_cloud):
    # Calculate the center by finding the average of all point coordinates
    num_points = len(point_cloud)
    center = np.mean(point_cloud, axis=0)
    return center


def rotate_point_cloud(point_cloud, rotation_matrix):
    # Translate point cloud to the origin (center)
    center = calculate_center(point_cloud)
    translated_point_cloud = point_cloud - center

    # Apply rotation to all points
    rotated_point_cloud = np.dot(translated_point_cloud, rotation_matrix)

    # Translate points back to their original positions
    rotated_point_cloud += center

    return rotated_point_cloud


def render_rotate_mesh(image_ori, point_cloud_data, delta_angle = 10, axis = 'y'):
    list_view = []
    for angle_degrees in range(-180, 180, delta_angle):
        angle_radians = np.radians(angle_degrees)
        
        if  axis == 'x':
            rotation_matrix = get_matrix_rotate_point_around_x(angle_radians)
        elif axis == 'y':
            rotation_matrix = get_matrix_rotate_point_around_y(angle_radians)
        elif axis == 'z':
            rotation_matrix = get_matrix_rotate_point_around_z(angle_radians)
        else: raise Exception(f'axis is invalid, please use `x`, `y` or `z`')

        # Rotate the point cloud
        rotated_point_cloud = rotate_point_cloud(point_cloud_data, rotation_matrix)
        new_mesh = render_mesh(image_ori, rotated_point_cloud, True)    
        list_view.append(new_mesh)
    return list_view



        
    

    




import cv2
import mediapipe as mp
import numpy as np

from scipy.spatial import Delaunay
from medface3D.utils.interpolation import bilinear_interpolation, bilinear_interpolation_triangle
from medface3D.utils.visualize import plot_mesh, plot_point_cloud
from medface3D.utils.transform import min_max_scale


class FaceMeshGenerator:
    def __init__(self, max_loop = 2, 
                detection_conf = 0.2, 
                face_mesh_dect_conf = 0.2,
                min_tracking_conf = 0.2):
    
        self.mp_face_detection = mp.solutions.face_detection
        self.mp_face_mesh = mp.solutions.face_mesh

        self.face_detection = self.mp_face_detection.FaceDetection(min_detection_confidence = detection_conf)
        self.face_mesh = self.mp_face_mesh.FaceMesh(min_detection_confidence = face_mesh_dect_conf, 
                                                    min_tracking_confidence = min_tracking_conf)
        
        self.max_loop = max_loop
    

    def generate_face_mesh(self, image, point_cloud = False, depth_scale = False):
        try:
            points = []
            triangles = []
            depth_list = []
            face_detected = []

            image_rgb = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
            results_detection = self.face_detection.process(image_rgb)

            if not results_detection.detections:
                return image, face_detected, points, depth_list, triangles
            
            for detection in results_detection.detections:
                bboxC = detection.location_data.relative_bounding_box
                ih, iw, _ = image.shape
                x, y, w, h = int(bboxC.xmin * iw), int(bboxC.ymin * ih), int(bboxC.width * iw), int(bboxC.height * ih)
                face_detected.append(image[y:h, x:w])
    
                results_landmarks = self.face_mesh.process(image_rgb)
                if not results_landmarks.multi_face_landmarks:
                    continue
                
                for landmarks in results_landmarks.multi_face_landmarks:
                    landmarks_list = [(int(landmark.x * iw), int(landmark.y * ih)) for landmark in landmarks.landmark]
                    depth_list = [landmark.z  for landmark in landmarks.landmark]

                    # Perform Delaunay triangulation
                    points = np.array(landmarks_list)
                    triangles = Delaunay(points)

                    for _ in range(self.max_loop):
                        for simplex in triangles.simplices:
                            pt1, pt2, pt3 = points[simplex]

                            tri_center = bilinear_interpolation_triangle(A=pt1, B=pt2, C=pt3)
                            landmarks_list.append(tri_center)
                            triangle_depths = [depth_list[i] for i in simplex]
                            depth_list.append(bilinear_interpolation(triangle_depths))

                        # Perform Delaunay triangulation
                        points = np.array(landmarks_list)
                        triangles = Delaunay(points)

                    # Perform Delaunay triangulation
                    points = np.array(landmarks_list)
                    triangles = Delaunay(points)

                    # scaling depth list
                    if depth_scale:
                        depth_list = min_max_scale(depth_list)
                    else: depth_list = np.asarray(depth_list)

                    if point_cloud: 
                        image = plot_point_cloud(image, points, depth_list)
                    else: 
                        image = plot_mesh(image, points, triangles.simplices, depth_list)        
                return image, face_detected, points, depth_list, triangles         
        except Exception as e:
            raise e


import os
import cv2
import argparse
import numpy as np

from medface3D.worker.face_mesh_reconstruction import FaceMeshGenerator
from medface3D.utils.save_mesh import save_ply_mesh
from medface3D.utils.visualize import get_depth_map, save_images_as_gif
from medface3D.utils.render_mesh import render_mesh, render_rotate_mesh
from medface3D.utils.transform import get_3D_point_cloud

def main():
    # Parse command-line arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-f',  '--folder',      default='data/sample' , help='path to sample folder image')
    parser.add_argument('-l',  '--max_loop',    type=int, default= 2,   help='max loop to reconstruct 3D face mesh')
    parser.add_argument('-p',  '--point_cloud', action="store_true",    help='get image result as point cloud')
    parser.add_argument('-d',  '--depth_scale', action="store_true",    help='depth scale for depth value')
    parser.add_argument('-sv', '--save_mesh',   action="store_true",    help='save result as .ply file')
    parser.add_argument('-dm', '--depth_map',   action="store_true",    help='save result as depth map')
    parser.add_argument('-sf', '--face_dense',  action="store_true",    help='save result as dense map')
    parser.add_argument('-rm', '--render_mesh',  action="store_true",    help='render 3D mesh map', default=True)
    parser.add_argument('-mv', '--mesh_view',  action="store_true",    help='render 3D mesh map', default=True)
    parser.add_argument('-ar', '--axis_rotate',  default='y',    help='render 3D mesh map')


    args = parser.parse_args()

    files = os.listdir(args.folder)

    for _, file in enumerate(files):
        if file.split('.')[-1] not in ['jpg', 'png']:
            continue

        path_read = f'{args.folder}/{file}'
        image = cv2.imread(path_read)
        h, w, c = image.shape

        detector = FaceMeshGenerator(max_loop = args.max_loop)
        input_image = image.copy()
        output_image, face_detected, points, depth_list, triangles = detector.generate_face_mesh(input_image, args.point_cloud, args.depth_scale)

        if not face_detected:
            print(f'No face detected in is image')

        if args.face_dense:
            cv2.imwrite(f'{args.folder}/{file}_face_dense.jpg', output_image)

        if args.depth_map:
            depth_map = get_depth_map(input_image, points, triangles.simplices, depth_list)
            cv2.imwrite(f'{args.folder}/{file}_face_depth.jpg', depth_map)

        if args.save_mesh:
            points_3d = get_3D_point_cloud(points, depth_list)
            save_ply_mesh(points_3d,  triangles.simplices, f'{args.folder}/{file}_face_dense.ply')

        if args.render_mesh:
            point_cloud_data = get_3D_point_cloud(points, depth_list, w)
            mesh_image = render_mesh(image, point_cloud_data)
            cv2.imwrite(f'{args.folder}/{file}_face_mesh.jpg', mesh_image)


        if args.mesh_view:
            point_cloud_data = get_3D_point_cloud(points, depth_list, w)
            list_view = render_rotate_mesh(image, point_cloud_data)
            save_images_as_gif(list_view, f'{args.folder}/{file}_face_mesh_view.gif')



if __name__ == '__main__':
    # Example usage:
    main()



import numpy as np

def min_max_scale(value_list):
    max_value = max(value_list)
    min_value = min(value_list)
    scaled_list = [(x - min_value) / (max_value - min_value) for x in value_list]
    return np.asarray(scaled_list)

def get_matrix_rotate_point_around_x(angle_radians):
    rotation_matrix = np.array([
        [1, 0, 0],
        [0, np.cos(angle_radians), -np.sin(angle_radians)],
        [0, np.sin(angle_radians), np.cos(angle_radians)]
    ])
    return rotation_matrix

def get_matrix_rotate_point_around_y(angle_radians):
    rotation_matrix = np.array([
        [np.cos(angle_radians), 0, np.sin(angle_radians)],
        [0, 1, 0],
        [-np.sin(angle_radians), 0, np.cos(angle_radians)]
    ])
    return rotation_matrix


def get_matrix_rotate_point_around_z(angle_radians):
    rotation_matrix = np.array([
        [np.cos(angle_radians), -np.sin(angle_radians), 0],
        [np.sin(angle_radians), np.cos(angle_radians), 0],
        [0, 0, 1]
    ])
    return rotation_matrix


def get_3D_point_cloud(points, depth_list, const = 450):
    points_3d = []
    for i, (x,y) in enumerate(points):
        z = const / 2 - depth_list[i] * const
        points_3d.append([x, y, z])
    
    return np.asarray(points_3d)


import numpy as np
from scipy.spatial.transform import Rotation

def merge_action(point1, point2):
    # Compute the centroid of each point cloud
    centroid1 = np.mean(point1, axis=0)
    centroid2 = np.mean(point2, axis=0)

    # Compute the translation vector between the centroids
    translation_vector = centroid1 - centroid2

    # Calculate the rotation matrix between the two point clouds
    # Here we assume that the point clouds have the same number of points and are aligned
    # You can use more advanced techniques (e.g., ICP) for better alignment if needed
    H = np.dot(point1.T, point2)
    U, S, Vt = np.linalg.svd(H)
    R = np.dot(Vt.T, U.T)

    # Create a rotation object using scipy's Rotation class
    rotation = Rotation.from_matrix(R)

    # Apply the translation and rotation to each point in point2 individually
    point2_transformed = []
    for point in point2:
        transformed_point = rotation.apply(point - centroid2) + centroid1
        point2_transformed.append(transformed_point)

    # Convert the result back to a NumPy array
    point2_transformed = np.array(point2_transformed)
    return point2_transformed


def save_ply(points, file_path):
    header = """ply
format ascii 1.0
element vertex {}
property float x
property float y
property float z
end_header
""".format(len(points))

    with open(file_path, 'w') as f:
        f.write(header)
        for point in points:
            x, y, z = point
            f.write("{} {} {}\n".format(x, y, z))


def save_ply_mesh(vertices, faces, file_path):
    header = """ply
format ascii 1.0
element vertex {}
property float x
property float y
property float z
element face {}
property list uchar int vertex_indices
end_header
""".format(len(vertices), len(faces))

    with open(file_path, 'w') as f:
        f.write(header)

        for vertex in vertices:
            x, y, z = vertex
            f.write("{} {} {}\n".format(x, y, z))

        for face in faces:
            f.write("3 {} {} {}\n".format(face[0], face[1], face[2]))


import cv2
import numpy as np
from PIL import Image

from utils.transform import min_max_scale

def plot_mesh(image, points, triangle, depth_list):
    for idx, simplex in enumerate(triangle):
        pt1, pt2, pt3 = points[simplex]
        depth1, depth2, depth3 = depth_list[simplex]

        # Map depth to a color
        depth_color_1 = int((depth1 + 1) * 127.5)  # Assuming depths range from -1 to 1
        depth_color_1 = (depth_color_1, 0, 0)

        depth_color_2 = int((depth2 + 1) * 127.5)  # Assuming depths range from -1 to 1
        depth_color_2 = (depth_color_2, 0, 0)

        depth_color_3 = int((depth3 + 1) * 127.5)  # Assuming depths range from -1 to 1
        depth_color_3 = (depth_color_3, 0, 0)

        cv2.line(image, tuple(pt1), tuple(pt2), depth_color_1, 1)
        cv2.line(image, tuple(pt2), tuple(pt3), depth_color_2, 1)
        cv2.line(image, tuple(pt3), tuple(pt1), depth_color_3, 1)
    return image


def plot_point_cloud(image, points, depth_list):
    for idx, point in enumerate(points):
        x, y = point
        z = 255 - depth_list[idx] * 255
        depth_color = (z, 0, 0)
        cv2.circle(image, (x,y), 1, depth_color, -1)
    return image


def get_depth_color(depth_value):
    z = 255 - depth_value * 255
    depth_color = (z, 255, 255)
    return depth_color


def get_depth_map(image, points, triangle, depth_list):
    background = np.zeros((image.shape[0], image.shape[1], 1))
    depth_list = min_max_scale(depth_list)

    for _, simplex in enumerate(triangle):
        pt1, pt2, pt3 = points[simplex]
        depth1, depth2, depth3 = depth_list[simplex]

        depth_color_1 = get_depth_color(depth1)
        depth_color_2 = get_depth_color(depth2)
        depth_color_3 = get_depth_color(depth3)

        cv2.line(background, tuple(pt1), tuple(pt2), depth_color_1, 3)
        cv2.line(background, tuple(pt2), tuple(pt3), depth_color_2, 3)
        cv2.line(background, tuple(pt3), tuple(pt1), depth_color_3, 3)
    return np.concatenate([background] * 3, axis=-1)


def save_images_as_gif(image_list, output_filename, duration=400):
    # Convert NumPy arrays to PIL Images
    pil_images = [Image.fromarray(np.uint8(image[:, :, ::-1])) for image in image_list]

    pil_images[0].save(
        output_filename,
        save_all=True,
        append_images=pil_images[1:],
        duration=duration,
        loop=0
    )


import numpy as np

def bilinear_interpolation_triangle(A, B, C, u = 1/3, v = 1/3):
    # Calculate the weights for each vertex of the triangle
    w_A = u
    w_B = v
    w_C = 1 - u - v

    # Calculate the interpolated point using the barycentric coordinates
    x_interpolated = w_A * A[0] + w_B * B[0] + w_C * C[0]
    y_interpolated = w_A * A[1] + w_B * B[1] + w_C * C[1]

    return (int(x_interpolated), int(y_interpolated))


def bilinear_interpolation(list_values, u = 1/3, v = 1/3):
    if len(list_values) != 3:
        raise ValueError("Input list_values must contain exactly 3 values.")

    w_A = u
    w_B = v
    w_C = 1 - u - v

    x1, x2, x3 = list_values

    pt_interpolated = w_A * x1 + w_B * x2 + w_C * x3
    return pt_interpolated


import os
import cv2
import argparse
import numpy as np
import traceback


from medface3D.worker.face_mesh_reconstruction import FaceMeshGenerator
from medface3D.worker.face_pose_estimation import FacePoseEstimator

from medface3D.utils.render_mesh import render_mesh
from medface3D.utils.transform import get_3D_point_cloud


def main():
    # Parse command-line arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-v',  '--video_path',  default=0 , help='path to sample folder image')
    parser.add_argument('-l',  '--max_loop',    type=int, default=0 ,   help='max loop to reconstruct 3D face mesh')

    args = parser.parse_args()

    detector = FaceMeshGenerator(max_loop = args.max_loop)
    pose_estimator = FacePoseEstimator()

    cap = cv2.VideoCapture(args.video_path)

    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    size = (int(cap.get(cv2.CAP_PROP_FRAME_WIDTH)) * 3, int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT)))

    if args.video_path == 0:
        name_save = 'local.mp4'
    else: name_save = os.path.basename(args.video_path)

    out = cv2.VideoWriter(f'data/demo/mesh_{name_save}', fourcc, 10, size)


    if not cap.isOpened():
        print("Error: Could not open camera.")
        exit()

    while True:
        try:
            ret, frame = cap.read()

            # Check if the frame was captured successfully
            if not ret:
                print("Error: Could not read frame.")
                break

            input_image = frame
            output_image, face_detected, points, depth_list, _ = detector.generate_face_mesh(input_image, True, False)

            if face_detected:
                point_cloud_data = get_3D_point_cloud(points, depth_list)

                face_pose = pose_estimator.get_pose(point_cloud_data)
                mesh_blank = render_mesh(input_image, point_cloud_data)
                mesh_map = render_mesh(input_image, point_cloud_data, False)
                output_image = cv2.hconcat([frame, mesh_blank])
                output_image = cv2.hconcat([output_image, mesh_map])

            out.write(output_image)
            # Display the frame in a window
            cv2.imshow("Video Capture", output_image)

            # Exit the loop when the 'q' key is pressed (or any other key)
            if cv2.waitKey(1) & 0xFF == ord('q'):
                break
        except Exception as e:
            continue

    # Release the VideoCapture object and close the OpenCV window
    cap.release()
    out.release()
    cv2.destroyAllWindows()


if __name__ == '__main__':
    # Example usage:
    main()



from setuptools import setup, find_packages
from setuptools.command.install import install
import subprocess
import os

current_file_path = os.path.abspath(__file__)
current_folder_path = os.path.dirname(current_file_path)

with open(os.path.join(current_folder_path, 'README.md'), 'r', encoding='utf-8') as fh:
    long_description = fh.read()

class PostInstallCommand(install):
    """Post-installation for installation mode."""
    def run(self):
        install.run(self)
        subprocess.check_call(['pip', 'install', '-r', os.path.join(current_folder_path, 'requirements.txt')])

setup(
    name='medface3d',
    version='0.0.1',
    include_package_data=True,
    package_data={
        'medface3D': ['data/demo/*.jpg', 'data/demo/*.png', 'data/demo/*.gif'],  # Include all jpg, png, and gif files,
        'medface3D': ['data/sample/*.jpg', 'data/sample/*.png', 'data/sample/*.gif'],  # Include all jpg, png, and gif files
    },
    packages=find_packages(),
    description= "3D face reconstruction using mediapipe 3D Face mesh",
    long_description = long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        # List other pip dependencies here if any
    ],
    cmdclass={
        'install': PostInstallCommand,
    },
    python_requires='>=3.7',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ]
)



import numpy as np

class FacePoseEstimator:
    def __init__(self) -> None:
        pass

    def get_pose(self, point_cloud):
        centroid = np.mean(point_cloud, axis=0)
        covariance_matrix = np.cov(point_cloud - centroid, rowvar=False)
        eigenvalues, eigenvectors = np.linalg.eig(covariance_matrix)

        roll = np.degrees(np.arctan2(eigenvectors[1, 0], eigenvectors[0, 0]))
        pitch = np.degrees(np.arctan2(-eigenvectors[2, 0], np.sqrt(eigenvectors[2, 1] ** 2 + eigenvectors[2, 2] ** 2)))
        yaw = np.degrees(np.arctan2(eigenvectors[2, 1], eigenvectors[2, 2]))

        return yaw, pitch, roll



